<?php
/**
 * 27/07/2015
 * @author Tarso
 */

use \Zeedhi\Framework\DTO\Request\Filter;
use \Zeedhi\Framework\DTO\Response;

namespace Ecommerce\Controllers;

use Zeedhi\Framework\ORM\ConnectionFactory;

class Login extends \Zeedhi\Framework\Controller\Simple
{

    protected $loginService;
    protected $ParamGeralService;

    public function __construct(Ecommerce\Services\LoginServiceEcommerce $loginService)
    {
        $this->loginService = $loginService;
    }

    public function autenticar($request, $response)
    {
        try {

            //@todo mudar para request do tipo row ao invés de uma do tipo filter.
            $credenciais = $request->getFilterCriteria()->getConditions();
            $fornecedor = $credenciais[0]['value'];
            $email = $credenciais[1]['value'];
            $senha = $credenciais[2]['value'];

            list($dadosDaSessao, $filhosFornecedor, $paramgeral) = $this->loginService->autenticar($fornecedor, $email, $senha);

            $filhosFornecedor = $this->formataMascara($filhosFornecedor, $this->getObjectProviderType());

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('paramgeral', $paramgeral));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('fornecedor', $dadosDaSessao));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('filhosFornecedor', $filhosFornecedor));

        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function checaEmail($request, $response)
    {

        try {

            $email = $request->getFilterCriteria()->getConditions()[0]['value'];
            $emailChecado = $this->loginService->checaEmail($email);
            $fornecedor = $this->organizaDatasetFornecedor($emailChecado);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('fornecedor', $fornecedor));

        } catch (\Exception $e) {

            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));

        }

    }

    public function verificaCodigoRecuperacao($request, $response)
    {

        try {

            $email = $request->getFilterCriteria()->getConditions()[0]['value'];
            $codigoVerificacao = $request->getFilterCriteria()->getConditions()[1]['value'];
            $novaSenha = $request->getFilterCriteria()->getConditions()[2]['value'];
            $senhaAlterada = (array)$this->loginService->addNovaSenha($email, $codigoVerificacao, $novaSenha);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('senhaAlterada', $senhaAlterada));

        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function organizaDatasetFornecedor($fornecedores)
    {

        $fornecedor = array();
        $listaFornecedor = array();

        foreach ($fornecedores as $emailChecado) {
            $fornecedor['CDFORNECED'] = $emailChecado->getCdforneced();
            $fornecedor['CDRANDOMACESSO'] = $emailChecado->getCdrandomacesso();
            $fornecedor['CDSENHAWEB'] = $emailChecado->getCdsenhaweb();
            $fornecedor['DSLOGINECO'] = $emailChecado->getDslogineco();
            $fornecedor['IDACESSOAF'] = $emailChecado->getIdacessoaf();
            $fornecedor['IDACESSOCOT'] = $emailChecado->getIdacessocot();
            $fornecedor['IDACESSOCR'] = $emailChecado->getIdacessocr();

            array_push($listaFornecedor, $fornecedor);
        }

        return $listaFornecedor;
    }

    public function salvaSenha($request, $response)
    {

        try {

            $email = $request->getFilterCriteria()->getConditions()[0]['value'];
            $senha = $request->getFilterCriteria()->getConditions()[1]['value'];

            $this->loginService->alteraSenha($email, $senha);

        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function verificaLogado($request, $response)
    {

        try {

            $email = $request->getFilterCriteria()->getConditions()[0]['value'];
            $emailChecado = $this->loginService->buscaUsuarioLogado($email);
            $fornecedor = $this->organizaDatasetFornecedor($emailChecado);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('fornecedor', $fornecedor));

        } catch (\Exception $e) {

            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));

        }

    }

    public function formataMascara($rows, $object)
    {

        for ($i = 0; $i < count($rows); $i++) {
            $type = $rows[$i]['IDTPIJURFORN'];
            $rows[$i]['IDTPIJURFORN'] = $object[$type]['name'];
            $maskInscricao = $object[$type]['mask'];

            if ($maskInscricao) {
                for ($j = 0; $j < strlen($rows[$i]['NRINSJURFORN']); $j++) {
                    $maskInscricao[strpos($maskInscricao, "#")] = $rows[$i]['NRINSJURFORN'][$j];
                }

                $rows[$i]['NRINSJURFORN'] = $maskInscricao;
            }
        }

        return $rows;
    }

    public function getObjectProviderType()
    {

        return
            array(
                "F" => array(
                    "name" => "Física",
                    "mask" => "###.###.###-##"
                ),
                "J" => array(
                    "name" => "Jurídica",
                    "mask" => "##.###.###/####-##"
                ),
                "I" => array(
                    "name" => "Internacional",
                    "mask" => ""
                ),
                "O" => array(
                    "name" => "Outros",
                    "mask" => ""
                )
            );
    }

    public function validaSenha($request, $response)
    {
        try {
            $credenciais = $request->getFilterCriteria()->getConditions();

            $cdsenhaweb = $credenciais[0]['value'];
            $senha = $credenciais[1]['value'];

            $this->loginService->validaSenha($cdsenhaweb, $senha);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('validaSenha', array(true)));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function criptographyPassword($request, $response)
    {
        $params = $request->getFilterCriteria()->getConditions();
        $password = $params[0]['value'];
        $connectionFactory = new ConnectionFactory();
        $password = $connectionFactory->encrypt($password);
        $retorno = array();
        $retorno['criptography'] = $password;
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('criptographyPassRepo', array($retorno)));
    }

}