var engine = function(eventAggregator, ScreenService, MessageService, General, LoginController, ZhuAclController, restEngine, ApplicationContext, templateManager) {
  /* global Constants */
    var aggregatorEvents = {
        BEFORE_PROJECT_STARTUP: "beforeProjectStartup",
        ON_FIRST_LOGIN: "onFirstLogin",
        ON_CONTAINER_OPEN: "onContainerOpen",
        AUTH_SUCCESS: "authSuccess",
        LOGIN_SUCCESS: "loginSuccess"
    };
    var regularExpressions = {
        LETTERS: /((a|A))/g,
        LETTERS_AND_NUMBERS: /((a|A|9))/g
    };
    var messages = {
        userDontHaveUnits: "Usuário não possui unidades relacionadas"
    };
    var messageTypes = {
        ERROR: "E",
        ALERT: "W"
    };

    var dataSource;

    eventAggregator.subscribe(aggregatorEvents.ON_FIRST_LOGIN, authUser);

    eventAggregator.subscribe(aggregatorEvents.BEFORE_PROJECT_STARTUP, authUser);

    eventAggregator.subscribe(aggregatorEvents.AUTH_SUCCESS, onAuthSuccess);

    function authUser() {
        LoginController.authenticate()
            .then(validateLoginResponse)
            .catch(callbackError);
    }
    
    function setUserId(user) {
		var userId = 'TF_' + user.cdoperador;
		if (ApplicationContext.ZhPreferences) {
			ApplicationContext.ZhPreferences.setUserID(userId);
		} else {
			ApplicationContext.zhCustomizationsService.registerUser(userId);
		}
	}

    function onAuthSuccess(user) {

        setUserLocalVars(user);
        setUserId(user);
        setGeneralParameters(user.nrorg).then(function() {}, callbackError);
        
        if (!getLocalVar('CDFORNECED')) {
            validateSuppliers(user.dsemailoper).then(function() {
                setParameters();
            }.bind(this), callbackError);
        } else {
            setMenuVisibility();
            setParameters();
        }
    }
    
    function controlVisibilityMenu(menuName, visibility) {
        const menu = templateManager.project.getMenu(menuName);
        
        if (typeof(menu) != "undefined") {
            menu.isVisible = visibility;
        }
    }
    
    function setMenuVisibility() {
        General.callRestEngine("product#/verificaIdsisdos", "DataSet", []).then(function(callback) {
            const idSisDos = callback.dataset.resp.IDSISDOS;
            let intervalMenu = setInterval(function(){
                if (templateManager.project.groupMenu[0].menus.length > 0) {
                    clearInterval(intervalMenu);
                    setVisibilityAdmin();
                    removeDefaultMenu();
                    
                    const components = getComponentsAccess(idSisDos);
                    
                    for (let component of components) {
                        if (component.isVisible) {
                            controlVisibilityMenu(component.menuName, true);
                        } else {
                            controlVisibilityMenu(component.menuName, false);
                        }
                        
                        if (component.widgetName == 'contasReceber') {
            				templateManager.project.removeMenu(component.removeMenuName);
            			}
                    }
                }
            }, 500);
        });
    }
    
    function setVisibilityAdmin() {
		const adminAccess = accessControl('IDADMINISTRADOR');
		
		if(!adminAccess) {
			templateManager.project.removeMenu('administrador');
		}
	}
	
	function removeDefaultMenu() {
		templateManager.project.removeMenu('parametrizacao');
		templateManager.project.removeMenu('acl');
		templateManager.project.removeMenu('log');
		templateManager.project.removeMenu('zhHistory');
	};
	
	function accessControl(access) {
		if (getLocalVar(access) == 'S') {
			return true;
		}
		
		return false;
	}
	
	function getComponentsAccess(idSisDos) {
		const accessPC = accessControl('IDACESSOCOT');
		const accessAF = accessControl('IDACESSOAF');
		const accessCR = accessControl('IDACESSOCR');
	    const accessMP = accessControl('IDACESSOUPLOAD');
		
		checkManualExist();
		
		let menuNameCR = "contas_a_receber";
		let windowNameCR = "product#ContasAReceberTek";
		let methodCR = 'dadosCR';
		let removeMenuNameCR = "contas_a_receber_GR";
		
		if(idSisDos == 3) {
			menuNameCR = "contas_a_receber_GR";
			windowNameCR = "product#ContasAReceber";
			methodCR = 'dadosContaGR';
			removeMenuNameCR = "contas_a_receber";
		}
		
		return [
			{
				"widgetName": "pedidoDeCotacao",
				"menuName": "pedidos_de_cotacao",
				"windowName": "product#Pedido_Cotacao",
				"isVisible": accessPC,
				"method": 'dadosPedCol'
			},
			{
				"widgetName": "autoFornecimento",
				"menuName": "autorizacao_de_fornecimento",
				"windowName": "product#AutorizacaoFornecimento",
				"isVisible": accessAF,
				"method": 'dadosAFNLidas'
			},
			{
				"widgetName": "contasReceber",
				"menuName": menuNameCR,
				"windowName": windowNameCR,
				"isVisible": accessCR,
				"method": methodCR,
				"removeMenuName": removeMenuNameCR
			},
			{
				"widgetName": "meusProdutos",
				"menuName": "meus_produtos",
				"windowName": "product#MeusProdutos",
				"isVisible": accessMP,
				"method": 'dadosMeusProdutos'
			}
		];
	};

    function validateLoginResponse(response) {
        var defaultResponse = response;
        response = LoginController.changeLoginResponseToOldParams(response);
        if (response && response.cdoperador) {
            startSession(response).then(function() {
                eventAggregator.publish(aggregatorEvents.AUTH_SUCCESS, response);
                eventAggregator.publish(aggregatorEvents.LOGIN_SUCCESS, defaultResponse);
                // setAcl(response);
            });
        }
    }

    // function setAcl(response) {
    //     var userId = response.cdoperador;
    //     var accessProfiles = response.perfilDeAcesso || [];
    //     var options = {
    //         defaultRule: "A"
    //     };
    //     ZhuAclController.setUserRules(accessProfiles, Config.loginProductId, userId, response.nrorg, options);
    //     setLocalVar("userId", userId);
    //     setLocalVar("accessProfiles", accessProfiles);
    //     setLocalVar("productIdLoginUrl", Config.loginProductId);
    // }

    function startSession(user) {
        return General.callRestEngine("product#/startSession", "Row", user).then(null, callbackError);
    }

    function setUserLocalVars(user) {
        setLocalVar('CDOPERADOR', user.cdoperador);
        setLocalVar('NMOPERADOR', user.nmoperador);
        setLocalVar('CDFILIAL', user.cdfilialselecionada);
        setLocalVar('NMFILIAL', user.nmfilialselecionada);
        setLocalVar('NRORG', user.nrorg);
        setLocalVar('NRORGTRAB', user.nrorg);
        setLocalVar('EMAIL', user.dsemailoper);
    }

    function replaceMask(value, regex) {
        return typeof(value) === 'string' ? value.replace(regex, '*') : value;
    }

    function setGeneralParameters(nrorg) {
        return getParamgeral().then(function(paramgeral) {
            if (paramgeral) {
                setLocalVar('CAD_CDPICTPRAT', replaceMask(paramgeral.CDPICTPRAT, regularExpressions.LETTERS));
                setLocalVar('GER_CDPICTPROD', replaceMask(paramgeral.CDPICTPROD, regularExpressions.LETTERS));
                setLocalVar('CDPICTPROD', replaceMask(paramgeral.CDPICTPROD, regularExpressions.LETTERS));
                setLocalVar('CDPICTCCTB', replaceMask(paramgeral.CDPICTCCTB, regularExpressions.LETTERS));
                setLocalVar('GER_CDPICTPRAT', replaceMask(paramgeral.CDPICTPRAT, regularExpressions.LETTERS_AND_NUMBERS));
                setLocalVar('CDPICTPRAT', replaceMask(paramgeral.CDPICTPRAT, regularExpressions.LETTERS_AND_NUMBERS));
            }
            else {
                MessageService.showMessageByCode("unsetGeneralParams", null, Constants.errorMessage);
            }
        });
    }

    function getParamgeral() {
        return getTableByName("paramgeral").then(function(response) {
            var paramgeral = null;
            if (response.length)
                paramgeral = response[0];
            return paramgeral;
        });
    }

    function getTableByName(tableName, filter) {
        if (!filter) {
            filter = [];
        }
        return General.callRestEngine("product#/crud/" + tableName, "FilterData", filter).then(function(response) {
            var result = [];
            if (response.dataset[tableName])
                result = response.dataset[tableName];
            return result;
        });
    }

    function callbackError(err) {
        if (err.data && err.data.error) {
            ScreenService.showMessage(err.data.error, messageTypes.ERROR);
        }
        else {
            ScreenService.showMessage(JSON.stringify(err), messageTypes.ERROR);
        }
    }

    this.logout = function() {
        let config = {
			type: "question",
			icon: "question",
			success: disconnect
		};
        MessageService.confirmMessageByCode("logout", null, config);
    };

    function disconnect() {
        removeLocalVars();
        LoginController.logout();
    }

    function removeLocalVars() {
        localforage.clear();
        localStorage.clear();
    }

    var buildUserAndHeader = function(name, descriptions, headerInfo) {

        ScreenService.buildUserData(name, descriptions, function() {
            ScreenService.openWindow('product#UserInfo');
        });

        ScreenService.setHeaderInfo(headerInfo);
    };

    function validateSuppliers(dsemailoper) {
        return getSupplier(dsemailoper).then(function(suppliers) {
            suppliers = suppliers.dataset.fornecedoresLogin;
            if (suppliers.length > 2) {
                ScreenService.openWindow('product#FornecedoresLogin');
                setLocalVar('INICIA_DASHBOARD', false);
            } else if (suppliers.length == 2) {
                setSupplierLocalVars(suppliers[1]);
                setMenuVisibility();
                setParameters();
                setLocalVar('INICIA_DASHBOARD', true);
            } else {
                setLocalVar('INICIA_DASHBOARD', false);
					let config = {
                    type: "alert",
                    icon: "exclamation",
                    actions: [
            			{
            				label: ScreenService.i18n('OK'),
            				code: function(e, deferred) {
                                if (Util.isFunction(disconnect)) {
                                    disconnect(e);
                                }
                                deferred.resolve(e);
                            }
            			}
            		]
                };
        		MessageService.showCustomDialogByCode('unregisteredSupplier', null, config);
            }
        });
    }

    var getSupplier = this.getSupplier = function(email) {
        return restEngine.post('product#/fornecedoresLogin', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "DSLOGINECO",
                    "operator": "=",
                    "value": email
                }
            ]
        });
    }

    function checkSelectedSupplier(widgetName) {
        var supplier = getLocalVar('CDFORNECED');
        if (!supplier && widgetName != 'WidgetMain') {
            ScreenService.openWindow('product#FornecedoresLogin');
            MessageService.showMessageByCode("chooseSupplier", null, Constants.alertMessage);
        }
    }
    
    function getDefaultContainers() {
        return [
            "product#dashboard",
            "product#Pedido_Cotacao",
            "product#AutorizacaoFornecimento",
            "product#ContasAReceber",
            "product#ContasAReceberTek",
            "product#MeusProdutos"
        ];
    }
    
    function clearContainers() {
        const containers = getDefaultContainers();
        
        for (let container of containers) {
            if (container in templateManager.containers) {
                delete templateManager.containers[container];
            }
        }
    }

    this.chooseSupplier = function(supplier) {

        if (!supplier._hasChild) {
            clearContainers();
            setLocalVar('INICIA_DASHBOARD', true);
            ScreenService.openWindow('product#dashboard');
            setSupplierLocalVars(supplier);
            setMenuVisibility();
            setParameters();
        }
    };

    /**
     * Questiona o usuário se ele realmente deseja alterar o fornecedor
     * relacionado no sistema.
     * Caso o sistema ainda não tenha um fornecedor definido, o sistema
     * altera automaticamente.
     * 
     * @param {object} supplier fornecedor selecionado.
     */
    this.checkSystemChangingAboutSelectedSupplier = function(supplier) {
        if (supplier.CDFORNECED != 1) {
            
            var currentSupplierCode = getLocalVar("CDFORNECED");
    
            if (currentSupplierCode) {
    
                if (currentSupplierCode === supplier.CDFORNECED) {
                    MessageService.showMessageByCode("authenticatedSupplier", null, Constants.alertMessage);
                }
                else {
                    let variables = {
                        NMRAZSOCFORN: getLocalVar("NMRAZSOCFORN"),
                        NEWNMRAZSOCFORN: supplier.NMRAZSOCFORN
                    };
                    MessageService.confirmMessageByCode('changeSupplierEngine', variables, Constants.confirmationMessage).then(function() {
                            this.chooseSupplier(supplier);
                        }.bind(this)
                    );
    
                }
    
            }
            else {
                this.chooseSupplier(supplier);
            }
            
        }

    };

    var setSupplierLocalVars = function(supplier) {
        setLocalVar('CDFORNECED', supplier.CDFORNECED);
        setLocalVar('CDFORNERELACAF', supplier.CDFORNERELACAF);
        setLocalVar('IDACESSOCOT', supplier.IDACESSOCOT);
        setLocalVar('IDACESSOAF', supplier.IDACESSOAF);
        setLocalVar('IDACESSOCR', supplier.IDACESSOCR);
        setLocalVar('IDACESSOUPLOAD', supplier.IDACESSOUPLOAD);
        setLocalVar('IDADMINISTRADOR', supplier.IDADMINISTRADOR);
        setLocalVar('NRINSJURFORN', supplier.NRINSJURFORN);
        setLocalVar('NMFANTFORN', supplier.NMFANTFORN);
        setLocalVar('NMRAZSOCFORN', supplier.NMRAZSOCFORN);
        setLocalVar('NMUSUARIOECO', supplier.NMUSUARIOECO);
        setLocalVar('DSLOGINECO', supplier.DSLOGINECO);
        setLocalVar('NRORG', supplier.NRORG);
        setLocalVar('NRORGTRAB', supplier.NRORG);
    };

    var setParameters = function() {

        var nrinsjurforn = getLocalVar('NRINSJURFORN');
        var nmfantforn = getLocalVar('NMFANTFORN');
        var nmrazsocforn = getLocalVar('NMRAZSOCFORN');
        var dslogineco = getLocalVar('DSLOGINECO');
        var nmusuarioeco = getLocalVar('NMUSUARIOECO');
        var nmoperador = getLocalVar('NMOPERADOR');
        
        nmusuarioeco = nmusuarioeco ? nmusuarioeco : nmoperador ? nmoperador : nmfantforn;
        
        var headerInfo = createHeaderInfo(nrinsjurforn, nmfantforn, nmrazsocforn);
        
        buildUserAndHeader(nmusuarioeco, [dslogineco], headerInfo);

    };

    /**
     * Verifica se existe um fornecedor autenticado no sitema, se existir, a função retorna
     * uma string apontando o fornecedor atual, mas caso ele não tenha, a função retorna
     * uma string vazia.
     * 
     * @param {string} NRINSJURFORN inscrição do fornecedor.
     * 
     * @param {string} NMFANTFORN nome fantasia do fornecedor.
     * 
     * @param {string} NMRAZSOCFORN Razão social do fornecedor.
     * 
     * @return {string} string referenciando o fornecedor em caso.
     */
    function createHeaderInfo(NRINSJURFORN, NMFANTFORN, NMRAZSOCFORN) {

        if (NRINSJURFORN) {
            return NRINSJURFORN + " - " + NMFANTFORN + " | " + NMRAZSOCFORN;
        }
        else {
            return "";
        }

    };
    
    /**
     * Correção de erro do zeedhi que não exibie dados do dataSource após filtrar
     * com "searchAction". Após o correção do erro, trecho poderá ser removido.
    **/
    this.reloadOnChangeDataSource = function(widget) {
        if (!isEqual(this.dataSource, widget.dataSource.data)) {
            this.dataSource = widget.dataSource.data;
            widget.reload();
        }
    };

    var isEqual = function(array1, array2) {
        if (Object.prototype.toString.call(array1) !== Object.prototype.toString.call(array2)) return false;

        if (Object.keys(array1).length !== Object.keys(array2).length) return false;

        Object.keys(array1).forEach(function(value, index) {
            if (!compareArray(array1[index], array2[index])) return false;
        }.bind(this));

        return true;
    };

    var compareArray = function(item1, item2) {
        if (['[object Array]', '[object Object]'].indexOf(Object.prototype.toString.call(item1)) >= 0) {
            if (!isEqual(item1, item2)) return false;
        }
        else {
            if (Object.prototype.toString.call(item1) !== Object.prototype.toString.call(item2)) return false;

            if (Object.prototype.toString.call(item1) === '[object Function]') {
                if (item1.toString() !== item2.toString())
                    return false;
            }
            else {
                if (item1 !== item2)
                    return false;
            }
        }
    };

    function checkManualExist() {
        General.callRestEngine("product#/checkManual", "DataSet", []).then(function(callback) {
            const path = callback.dataset.resp[0];
            
            if (path != "") {
                portalManualButton(templateManager.project, true);
            } else {
                portalManualButton(templateManager.project, false);
            }
        });
    }

    function portalManualButton(project, isVisible) {
        let portalManual = {
			"label": "Manual do Portal",
			"name": "portalManual",
			"icon": "download",
			"isVisible": isVisible,
			"tooltip": "Baixar Manual",
			"isAclNotification": true,
			"events": [
				{
					"name": "NotificationEvent",
					"code": "engine.downloadManual();"
				}
			]
		};
		
		try {
		    const button = checkButtonExist(project.notifications);
			
			if (!button) {
    			portalManual = Util.extend(new _Notification(), portalManual);
    			project.notifications = project.notifications || [];
    			project.notifications.push(portalManual);
			}
		} catch (e){
			console.error('O Controle de acesso à elementos só funciona em versões acima da 1.40.x do Zeedhi Framework');
		}
    }
    
    function checkButtonExist(notifications) {
        for (let notification of notifications) {
            if (notification.name == "portalManual") {
                return true;
            }
        }
        
        return false;
    };
    
    this.downloadManual = function() {
        General.callRestEngine("product#/checkManual", "DataSet", []).then(function(callback) {
            const path = callback.dataset.resp[0];
            
            if (path != "") {
                var mappedPackages = templateManager.getMappedServicePackages();
                var baseUrl = mappedPackages.product;
                window.open(baseUrl + "/downloadManual?requestType=DataSet&dataset[0][path]=" + path);
            }
        });
    }
};

Configuration(function(ContextRegister) {
    ContextRegister.register('engine', engine);
});